@@ -37,7 +37,9 @@ def json_converter(obj):
 class CounterfactualExamples:
     """A class to store and visualize the resulting counterfactual explanations."""
 
-    def __init__(self, data_interface=None, final_cfs_df=None, test_instance_df=None, final_cfs_df_sparse=None, posthoc_sparsity_param=0, desired_range=None, desired_class="opposite", model_type='classifier'):
+    def __init__(self, data_interface=None, final_cfs_df=None, test_instance_df=None,
+                 final_cfs_df_sparse=None, posthoc_sparsity_param=0,
+                 desired_range=None, desired_class="opposite", model_type='classifier'):
 
         self.data_interface = data_interface
         self.final_cfs_df = final_cfs_df
@@ -48,7 +50,7 @@ class CounterfactualExamples:
         self.desired_range = desired_range
 
         self.final_cfs_list = None
-        self.posthoc_sparsity_param = posthoc_sparsity_param # might be useful for future additions
+        self.posthoc_sparsity_param = posthoc_sparsity_param  # might be useful for future additions
 
         self.test_pred = self.test_instance_df[self.data_interface.outcome_name].iat[0]
         if model_type == 'classifier':
@@ -68,25 +70,28 @@ class CounterfactualExamples:
     def visualize_as_dataframe(self, display_sparse_df=True, show_only_changes=False):
         from IPython.display import display
         # original instance
-        print('Query instance (original outcome : %i)' %round(self.test_pred))
-        display(self.test_instance_df) #  works only in Jupyter notebook
+        print('Query instance (original outcome : %i)' % round(self.test_pred))
+        display(self.test_instance_df)  # works only in Jupyter notebook
         if self.final_cfs_df is not None and len(self.final_cfs_df) > 0:
-            if self.posthoc_sparsity_param == None:
+            if self.posthoc_sparsity_param is None:
                 print('\nCounterfactual set (new outcome: {0})'.format(self.new_outcome))
                 self.display_df(self.final_cfs_df, show_only_changes)
-
-            elif hasattr(self.data_interface, 'data_df') and display_sparse_df==True and self.final_cfs_df_sparse is not None:
+            elif hasattr(self.data_interface, 'data_df') and \
+                    display_sparse_df is True and self.final_cfs_df_sparse is not None:
                 # CFs
                 print('\nDiverse Counterfactual set (new outcome: {0})'.format(self.new_outcome))
                 self.display_df(self.final_cfs_df_sparse, show_only_changes)
 
-
-            elif hasattr(self.data_interface, 'data_df') and display_sparse_df==True and self.final_cfs_df_sparse is None:
-                print('\nPlease specify a valid posthoc_sparsity_param to perform sparsity correction.. displaying Diverse Counterfactual set without sparsity correction (new outcome : %i)' %(self.new_outcome))
+            elif hasattr(self.data_interface, 'data_df') and \
+                    display_sparse_df is True and self.final_cfs_df_sparse is None:
+                print('\nPlease specify a valid posthoc_sparsity_param to perform sparsity correction.. ',
+                      'displaying Diverse Counterfactual set without sparsity correction (new outcome : %i)' %
+                      (self.new_outcome))
                 self.display_df(self.final_cfs_df, show_only_changes)
 
-            elif not hasattr(self.data_interface, 'data_df'):# for private data
-                print('\nDiverse Counterfactual set without sparsity correction since only metadata about each feature is available (new outcome: ', self.new_outcome)
+            elif not hasattr(self.data_interface, 'data_df'):  # for private data
+                print('\nDiverse Counterfactual set without sparsity correction since only metadata about each',
+                      ' feature is available (new outcome: ', self.new_outcome)
                 self.display_df(self.final_cfs_df, show_only_changes)
 
             else:
@@ -113,30 +118,35 @@ class CounterfactualExamples:
 
     def visualize_as_list(self, display_sparse_df=True, show_only_changes=False):
         # original instance
-        print('Query instance (original outcome : %i)' %round(self.test_pred))
+        print('Query instance (original outcome : %i)' % round(self.test_pred))
         print(self.test_instance_df.values.tolist()[0])
 
         if len(self.final_cfs) > 0:
-            if self.posthoc_sparsity_param == None:
-                print('\nCounterfactual set (new outcome : %i)' %(self.new_outcome))
+            if self.posthoc_sparsity_param is None:
+                print('\nCounterfactual set (new outcome : %i)' % (self.new_outcome))
                 self.print_list(self.final_cfs_df.values.tolist(), show_only_changes)
 
-            elif hasattr(self.data_interface, 'data_df') and display_sparse_df==True and self.final_cfs_df_sparse is not None:
+            elif hasattr(self.data_interface, 'data_df') and \
+                    display_sparse_df is True and self.final_cfs_df_sparse is not None:
                 # CFs
-                print('\nDiverse Counterfactual set (new outcome : %i)' %(self.new_outcome))
+                print('\nDiverse Counterfactual set (new outcome : %i)' % (self.new_outcome))
                 self.print_list(self.final_cfs_df_sparse.values.tolist(), show_only_changes)
 
-            elif hasattr(self.data_interface, 'data_df') and display_sparse_df==True and self.final_cfs_df_sparse is None:
-                print('\nPlease specify a valid posthoc_sparsity_param to perform sparsity correction.. displaying Diverse Counterfactual set without sparsity correction (new outcome : %i)' %(self.new_outcome))
+            elif hasattr(self.data_interface, 'data_df') and \
+                    display_sparse_df is True and self.final_cfs_df_sparse is None:
+                print('\nPlease specify a valid posthoc_sparsity_param to perform sparsity correction.. ',
+                      'displaying Diverse Counterfactual set without sparsity correction (new outcome : %i)'
+                      % (self.new_outcome))
                 self.print_list(self.final_cfs_df.values.tolist(), show_only_changes)
 
-            elif not hasattr(self.data_interface, 'data_df'): # for private data
-                print('\nDiverse Counterfactual set without sparsity correction since only metadata about each feature is available (new outcome : %i)' %(self.new_outcome))
+            elif not hasattr(self.data_interface, 'data_df'):  # for private data
+                print('\nDiverse Counterfactual set without sparsity correction since only metadata about ',
+                      'each feature is available (new outcome : %i)' % (self.new_outcome))
                 self.print_list(self.final_cfs_df.values.tolist(), show_only_changes)
 
             else:
                 # CFs
-                print('\nDiverse Counterfactual set without sparsity correction (new outcome : %i)' %(self.new_outcome))
+                print('\nDiverse Counterfactual set without sparsity correction (new outcome : %i)' % (self.new_outcome))
                 self.print_list(self.final_cfs_df.values.tolist(), show_only_changes)
         else:
             print('\n0 counterfactuals found!')
