@@ -20,46 +20,54 @@ class ExplainerBase:
         :param data_interface: an interface class to access data related params.
         :param model_interface: an interface class to access trained ML model.
         """
-
         # initiating data and model related parameters
         self.data_interface = data_interface
         if model_interface is not None:
-            #self.data_interface.create_ohe_params()
+            # self.data_interface.create_ohe_params()
             self.model = model_interface
-            self.model.load_model() # loading pickled trained model if applicable
+            self.model.load_model()  # loading pickled trained model if applicable
             self.model.transformer.feed_data_params(data_interface)
             self.model.transformer.initialize_transform_func()
 
         # moved the following snippet to a method in public_data_interface
-                # self.minx, self.maxx, self.encoded_categorical_feature_indexes = self.data_interface.get_data_params()
-                #
-                # # min and max for continuous features in original scale
-                # flattened_indexes = [item for sublist in self.encoded_categorical_feature_indexes for item in sublist]
-                # self.encoded_continuous_feature_indexes = [ix for ix in range(len(self.minx[0])) if ix not in flattened_indexes]
-                # org_minx, org_maxx = self.data_interface.get_minx_maxx(normalized=False)
-                # self.cont_minx = list(org_minx[0][self.encoded_continuous_feature_indexes])
-                # self.cont_maxx = list(org_maxx[0][self.encoded_continuous_feature_indexes])
-                #
-                # # decimal precisions for continuous features
-                # self.cont_precisions = [self.data_interface.get_decimal_precisions()[ix] for ix in self.encoded_continuous_feature_indexes]
+        # self.minx, self.maxx, self.encoded_categorical_feature_indexes = self.data_interface.get_data_params()
+        #
+        # # min and max for continuous features in original scale
+        # flattened_indexes = [item for sublist in self.encoded_categorical_feature_indexes for item in sublist]
+        # self.encoded_continuous_feature_indexes = [ix for ix in range(len(self.minx[0])) if ix not in flattened_indexes]
+        # org_minx, org_maxx = self.data_interface.get_minx_maxx(normalized=False)
+        # self.cont_minx = list(org_minx[0][self.encoded_continuous_feature_indexes])
+        # self.cont_maxx = list(org_maxx[0][self.encoded_continuous_feature_indexes])
+        #
+        # # decimal precisions for continuous features
+        # self.cont_precisions = \
+        #   [self.data_interface.get_decimal_precisions()[ix] for ix in self.encoded_continuous_feature_indexes]
 
     def generate_counterfactuals(self, query_instances, total_CFs,
-            desired_class="opposite", desired_range=None,
-            permitted_range=None, features_to_vary="all",
-            stopping_threshold=0.5, posthoc_sparsity_param=0.1,
-            posthoc_sparsity_algorithm="linear", verbose=False, **kwargs):
+                                 desired_class="opposite", desired_range=None,
+                                 permitted_range=None, features_to_vary="all",
+                                 stopping_threshold=0.5, posthoc_sparsity_param=0.1,
+                                 posthoc_sparsity_algorithm="linear", verbose=False, **kwargs):
         """General method for generating counterfactuals.
 
-        :param query_instances: Input point(s) for which counterfactuals are to be generated. This can be a dataframe with one or more rows.
+        :param query_instances: Input point(s) for which counterfactuals are to be generated.
+                                This can be a dataframe with one or more rows.
         :param total_CFs: Total number of counterfactuals required.
 
-        :param desired_class: Desired counterfactual class - can take 0 or 1. Default value is "opposite" to the outcome class of query_instance for binary classification.
-        :param desired_range: For regression problems. Contains the outcome range to generate counterfactuals in.
-        :param permitted_range: Dictionary with feature names as keys and permitted range in list as values. Defaults to the range inferred from training data. If None, uses the parameters initialized in data_interface.
+        :param desired_class: Desired counterfactual class - can take 0 or 1. Default value
+                              is "opposite" to the outcome class of query_instance for binary classification.
+        :param desired_range: For regression problems. Contains the outcome range to
+                              generate counterfactuals in.
+        :param permitted_range: Dictionary with feature names as keys and permitted range in list as values.
+                                Defaults to the range inferred from training data.
+                                If None, uses the parameters initialized in data_interface.
         :param features_to_vary: Either a string "all" or a list of feature names to vary.
         :param stopping_threshold: Minimum threshold for counterfactuals target class probability.
         :param posthoc_sparsity_param: Parameter for the post-hoc operation on continuous features to enhance sparsity.
-        :param posthoc_sparsity_algorithm: Perform either linear or binary search. Takes "linear" or "binary". Prefer binary search when a feature range is large (for instance, income varying from 10k to 1000k) and only if the features share a monotonic relationship with predicted outcome in the model.
+        :param posthoc_sparsity_algorithm: Perform either linear or binary search. Takes "linear" or "binary".
+                                           Prefer binary search when a feature range is large (for instance,
+                                           income varying from 10k to 1000k) and only if the features share a
+                                           monotonic relationship with predicted outcome in the model.
         :param verbose: Whether to output detailed messages.
         :param sample_size: Sampling size
         :param random_seed: Random seed for reproducibility
@@ -69,7 +77,8 @@ class ExplainerBase:
         counterfactual examples per query_instance as one of its attributes.
         """
         if total_CFs <= 0:
-            raise UserConfigValidationException("The number of counterfactuals generated per query instance (total_CFs) should be a positive integer.")
+            raise UserConfigValidationException(
+                "The number of counterfactuals generated per query instance (total_CFs) should be a positive integer.")
         cf_examples_arr = []
         query_instances_list = []
         if isinstance(query_instances, pd.DataFrame):
@@ -78,16 +87,17 @@ class ExplainerBase:
         elif isinstance(query_instances, Iterable):
             query_instances_list = query_instances
         for query_instance in tqdm(query_instances_list):
-            res = self._generate_counterfactuals(query_instance, total_CFs,
-                    desired_class=desired_class,
-                    desired_range=desired_range,
-                    permitted_range=permitted_range,
-                    features_to_vary=features_to_vary,
-                    stopping_threshold=stopping_threshold,
-                    posthoc_sparsity_param=posthoc_sparsity_param,
-                    posthoc_sparsity_algorithm=posthoc_sparsity_algorithm,
-                    verbose=verbose,
-                    **kwargs)
+            res = self._generate_counterfactuals(
+                query_instance, total_CFs,
+                desired_class=desired_class,
+                desired_range=desired_range,
+                permitted_range=permitted_range,
+                features_to_vary=features_to_vary,
+                stopping_threshold=stopping_threshold,
+                posthoc_sparsity_param=posthoc_sparsity_param,
+                posthoc_sparsity_algorithm=posthoc_sparsity_algorithm,
+                verbose=verbose,
+                **kwargs)
             cf_examples_arr.append(res)
         return CounterfactualExplanations(cf_examples_list=cf_examples_arr)
 
@@ -98,7 +108,7 @@ class ExplainerBase:
         if permitted_range is None:  # use the precomputed default
             self.feature_range = self.data_interface.permitted_range
             feature_ranges_orig = self.feature_range
-        else: # compute the new ranges based on user input
+        else:  # compute the new ranges based on user input
             self.feature_range, feature_ranges_orig = self.data_interface.get_features_range(permitted_range)
         self.check_query_instance_validity(features_to_vary, permitted_range, query_instance, feature_ranges_orig)
 
@@ -121,85 +131,124 @@ class ExplainerBase:
                         raise ValueError("Feature:", feature, "is outside the permitted range and isn't allowed to vary.")
 
     def local_feature_importance(self, query_instances, cf_examples_list=None,
-            total_CFs=10,
-            desired_class="opposite", desired_range=None, permitted_range=None,
-            features_to_vary="all", stopping_threshold=0.5,
-            posthoc_sparsity_param=0.1, posthoc_sparsity_algorithm="linear",
-            **kwargs):
+                                 total_CFs=10,
+                                 desired_class="opposite", desired_range=None, permitted_range=None,
+                                 features_to_vary="all", stopping_threshold=0.5,
+                                 posthoc_sparsity_param=0.1, posthoc_sparsity_algorithm="linear",
+                                 **kwargs):
         """ Estimate local feature importance scores for the given inputs.
 
-
         :param query_instances: A list of inputs for which to compute the
-        feature importances. These can be provided as a dataframe.
+                                feature importances. These can be provided as a dataframe.
         :param cf_examples_list: If precomputed, a list of counterfactual
-        examples for every input point. If cf_examples_list is provided, then
-        all the following parameters are ignored.
+                                 examples for every input point. If cf_examples_list is provided, then
+                                 all the following parameters are ignored.
         :param total_CFs: The number of counterfactuals to generate per input
-        (default is 10)
+                          (default is 10)
         :param other_parameters: These are the same as the
-        generate_counterfactuals method.
+                                 generate_counterfactuals method.
 
         :returns: An object of class CounterfactualExplanations that includes
-        the list of counterfactuals per input, local feature importances per
-        input, and the global feature importance summarized over all inputs.
+                  the list of counterfactuals per input, local feature importances per
+                  input, and the global feature importance summarized over all inputs.
         """
         if cf_examples_list is not None:
             if any([len(cf_examples.final_cfs_df) < 10 for cf_examples in cf_examples_list]):
-                raise UserConfigValidationException("The number of counterfactuals generated per query instance should be greater than or equal to 10")
+                raise UserConfigValidationException(
+                    "The number of counterfactuals generated per query instance should be "
+                    "greater than or equal to 10")
         elif total_CFs < 10:
-            raise UserConfigValidationException("The number of counterfactuals generated per query instance should be greater than or equal to 10")
-        importances = self.feature_importance(query_instances,
-                cf_examples_list=cf_examples_list,
-                total_CFs=total_CFs,
-                local_importance=True,
-                global_importance=False,
-                desired_class=desired_class,
-                desired_range=desired_range,
-                permitted_range=permitted_range,
-                features_to_vary=features_to_vary,
-                stopping_threshold=stopping_threshold,
-                posthoc_sparsity_param=posthoc_sparsity_param,
-                posthoc_sparsity_algorithm=posthoc_sparsity_algorithm,
-                **kwargs)
+            raise UserConfigValidationException("The number of counterfactuals generated per "
+                                                "query instance should be greater than or equal to 10")
+        importances = self.feature_importance(
+            query_instances,
+            cf_examples_list=cf_examples_list,
+            total_CFs=total_CFs,
+            local_importance=True,
+            global_importance=False,
+            desired_class=desired_class,
+            desired_range=desired_range,
+            permitted_range=permitted_range,
+            features_to_vary=features_to_vary,
+            stopping_threshold=stopping_threshold,
+            posthoc_sparsity_param=posthoc_sparsity_param,
+            posthoc_sparsity_algorithm=posthoc_sparsity_algorithm,
+            **kwargs)
         return importances
 
     def global_feature_importance(self, query_instances, cf_examples_list=None,
-            total_CFs=10, local_importance=True,
-            desired_class="opposite", desired_range=None, permitted_range=None,
-            features_to_vary="all", stopping_threshold=0.5,
-            posthoc_sparsity_param=0.1, posthoc_sparsity_algorithm="linear",
-            **kwargs):
+                                  total_CFs=10, local_importance=True,
+                                  desired_class="opposite", desired_range=None, permitted_range=None,
+                                  features_to_vary="all", stopping_threshold=0.5,
+                                  posthoc_sparsity_param=0.1, posthoc_sparsity_algorithm="linear",
+                                  **kwargs):
         """ Estimate global feature importance scores for the given inputs.
 
-
         :param query_instances: A list of inputs for which to compute the
-        feature importances. These can be provided as a dataframe.
+                                feature importances. These can be provided as a dataframe.
         :param cf_examples_list: If precomputed, a list of counterfactual
-        examples for every input point. If cf_examples_list is provided, then
-        all the following parameters are ignored.
+                                 examples for every input point. If cf_examples_list is provided, then
+                                 all the following parameters are ignored.
         :param total_CFs: The number of counterfactuals to generate per input
-        (default is 10)
+                          (default is 10)
         :param local_importance: Binary flag indicating whether local feature
-        importance values should also be returned for each query instance.
-        :param other_parameters: These are the same as the
-        generate_counterfactuals method.
+                                 importance values should also be returned for each query instance.
+        :param other_parameters: These are the same as the generate_counterfactuals method.
 
         :returns: An object of class CounterfactualExplanations that includes
-        the list of counterfactuals per input, local feature importances per
-        input, and the global feature importance summarized over all inputs.
+                  the list of counterfactuals per input, local feature importances per
+                  input, and the global feature importance summarized over all inputs.
         """
         if len(query_instances) < 10:
             raise UserConfigValidationException("The number of query instances should be greater than or equal to 10")
         if cf_examples_list is not None:
             if any([len(cf_examples.final_cfs_df) < 10 for cf_examples in cf_examples_list]):
-                raise UserConfigValidationException("The number of counterfactuals generated per query instance should be greater than or equal to 10")
+                raise UserConfigValidationException(
+                    "The number of counterfactuals generated per query instance should be "
+                    "greater than or equal to 10")
         elif total_CFs < 10:
-            raise UserConfigValidationException("The number of counterfactuals generated per query instance should be greater than or equal to 10")
-        importances = self.feature_importance(query_instances,
-                cf_examples_list=cf_examples_list,
-                total_CFs=total_CFs,
-                local_importance=local_importance,
-                global_importance=True,
+            raise UserConfigValidationException(
+                "The number of counterfactuals generated per query instance should be greater "
+                "than or equal to 10")
+        importances = self.feature_importance(
+            query_instances,
+            cf_examples_list=cf_examples_list,
+            total_CFs=total_CFs,
+            local_importance=local_importance,
+            global_importance=True,
+            desired_class=desired_class,
+            desired_range=desired_range,
+            permitted_range=permitted_range,
+            features_to_vary=features_to_vary,
+            stopping_threshold=stopping_threshold,
+            posthoc_sparsity_param=posthoc_sparsity_param,
+            posthoc_sparsity_algorithm=posthoc_sparsity_algorithm,
+            **kwargs)
+        return importances
+
+    def feature_importance(self, query_instances, cf_examples_list=None,
+                           total_CFs=10, local_importance=True, global_importance=True,
+                           desired_class="opposite", desired_range=None,
+                           permitted_range=None, features_to_vary="all", stopping_threshold=0.5,
+                           posthoc_sparsity_param=0.1, posthoc_sparsity_algorithm="linear", **kwargs):
+        """ Estimate feature importance scores for the given inputs.
+
+        :param query_instances: A list of inputs for which to compute the
+                                feature importances. These can be provided as a dataframe.
+        :param cf_examples_list: If precomputed, a list of counterfactual
+                                 examples for every input point. If cf_examples_list is provided, then
+                                 all the following parameters are ignored.
+        :param total_CFs: The number of counterfactuals to generate per input
+                          (default is 10)
+        :param other_parameters: These are the same as the generate_counterfactuals method.
+
+        :returns: An object of class CounterfactualExplanations that includes
+                  the list of counterfactuals per input, local feature importances per
+                  input, and the global feature importance summarized over all inputs.
+        """
+        if cf_examples_list is None:
+            cf_examples_list = self.generate_counterfactuals(
+                query_instances, total_CFs,
                 desired_class=desired_class,
                 desired_range=desired_range,
                 permitted_range=permitted_range,
@@ -207,41 +256,7 @@ class ExplainerBase:
                 stopping_threshold=stopping_threshold,
                 posthoc_sparsity_param=posthoc_sparsity_param,
                 posthoc_sparsity_algorithm=posthoc_sparsity_algorithm,
-                **kwargs)
-        return importances
-
-    def feature_importance(self, query_instances, cf_examples_list=None,
-            total_CFs=10, local_importance=True, global_importance=True,
-            desired_class="opposite", desired_range=None,
-            permitted_range=None, features_to_vary="all", stopping_threshold=0.5,
-            posthoc_sparsity_param=0.1, posthoc_sparsity_algorithm="linear", **kwargs):
-        """ Estimate feature importance scores for the given inputs.
-
-        :param query_instances: A list of inputs for which to compute the
-        feature importances. These can be provided as a dataframe.
-        :param cf_examples_list: If precomputed, a list of counterfactual
-        examples for every input point. If cf_examples_list is provided, then
-        all the following parameters are ignored.
-        :param total_CFs: The number of counterfactuals to generate per input
-        (default is 10)
-        :param other_parameters: These are the same as the
-        generate_counterfactuals method.
-
-        :returns: An object of class CounterfactualExplanations that includes
-        the list of counterfactuals per input, local feature importances per
-        input, and the global feature importance summarized over all inputs.
-        """
-
-        if cf_examples_list is None:
-            cf_examples_list = self.generate_counterfactuals(query_instances, total_CFs,
-                    desired_class=desired_class,
-                    desired_range=desired_range,
-                    permitted_range=permitted_range,
-                    features_to_vary=features_to_vary,
-                    stopping_threshold=stopping_threshold,
-                    posthoc_sparsity_param=posthoc_sparsity_param,
-                    posthoc_sparsity_algorithm=posthoc_sparsity_algorithm,
-                    **kwargs).cf_examples_list
+                **kwargs).cf_examples_list
         allcols = self.data_interface.categorical_feature_names + self.data_interface.continuous_feature_names
         summary_importance = None
         local_importances = None
@@ -287,25 +302,28 @@ class ExplainerBase:
         if summary_importance is not None:
             for col in allcols:
                 summary_importance[col] /= (cf_examples_list[0].final_cfs_df.shape[0]*len(cf_examples_list))
-        return CounterfactualExplanations(cf_examples_list,
-                local_importance=local_importances,
-                summary_importance=summary_importance)
-
+        return CounterfactualExplanations(
+            cf_examples_list,
+            local_importance=local_importances,
+            summary_importance=summary_importance)
 
     def predict_fn_for_sparsity(self, input_instance):
         """prediction function for sparsity correction"""
         return self.model.get_output(input_instance)
 
-    def do_posthoc_sparsity_enhancement(self, final_cfs_sparse, query_instance, posthoc_sparsity_param, posthoc_sparsity_algorithm):
+    def do_posthoc_sparsity_enhancement(self, final_cfs_sparse, query_instance, posthoc_sparsity_param,
+                                        posthoc_sparsity_algorithm):
         """Post-hoc method to encourage sparsity in a generated counterfactuals.
 
         :param final_cfs_sparse: Final CFs in original user-fed format, in a pandas dataframe.
         :param query_instance: Query instance in original user-fed format, in a pandas dataframe.
         :param posthoc_sparsity_param: Parameter for the post-hoc operation on continuous features to enhance sparsity.
-        :param posthoc_sparsity_algorithm: Perform either linear or binary search. Prefer binary search when a feature range is large (for instance, income varying from 10k to 1000k) and only if the features share a monotonic relationship with predicted outcome in the model.
+        :param posthoc_sparsity_algorithm: Perform either linear or binary search.
+                                           Prefer binary search when a feature range is
+                                           large (for instance, income varying from 10k to 1000k)
+                                           and only if the features share a monotonic relationship
+                                           with predicted outcome in the model.
         """
-        # :param total_random_inits: total random initialization required for algorithm = RandomInitCF (where k CFs are generated by running a CF generation algorithm k times with random initializations.)
-
         if final_cfs_sparse is None:
             return final_cfs_sparse
 
@@ -333,31 +351,34 @@ class ExplainerBase:
             current_pred = self.predict_fn_for_sparsity(
                 final_cfs_sparse.iloc[[cf_ix]][self.data_interface.feature_names])
             for feature in features_sorted:
-                #current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.iat[[cf_ix]][self.data_interface.feature_names])
-                #feat_ix = self.data_interface.continuous_feature_names.index(feature)
+                # current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.iat[[cf_ix]][self.data_interface.feature_names])
+                # feat_ix = self.data_interface.continuous_feature_names.index(feature)
                 diff = query_instance[feature].iat[0] - final_cfs_sparse[feature].iat[cf_ix]
                 if(abs(diff) <= quantiles[feature]):
                     if posthoc_sparsity_algorithm == "linear":
-                        final_cfs_sparse = self.do_linear_search(diff, decimal_prec, query_instance, cf_ix, feature, final_cfs_sparse, current_pred)
+                        final_cfs_sparse = self.do_linear_search(diff, decimal_prec, query_instance, cf_ix,
+                                                                 feature, final_cfs_sparse, current_pred)
 
                     elif posthoc_sparsity_algorithm == "binary":
-                        final_cfs_sparse = self.do_binary_search(diff, decimal_prec, query_instance, cf_ix, feature, final_cfs_sparse, current_pred)
+                        final_cfs_sparse = self.do_binary_search(
+                            diff, decimal_prec, query_instance, cf_ix, feature, final_cfs_sparse, current_pred)
 
             temp_preds = self.predict_fn_for_sparsity(final_cfs_sparse.iloc[[cf_ix]][self.data_interface.feature_names])
             cfs_preds_sparse.append(temp_preds)
 
         final_cfs_sparse[self.data_interface.outcome_name] = self.get_model_output_from_scores(cfs_preds_sparse)
-        #final_cfs_sparse[self.data_interface.outcome_name] = np.round(final_cfs_sparse[self.data_interface.outcome_name], 3)
+        # final_cfs_sparse[self.data_interface.outcome_name] = np.round(final_cfs_sparse[self.data_interface.outcome_name], 3)
         return final_cfs_sparse
 
     def do_linear_search(self, diff, decimal_prec, query_instance, cf_ix, feature, final_cfs_sparse, current_pred_orig):
-        """Performs a greedy linear search - moves the continuous features in CFs towards original values in query_instance greedily until the prediction class changes."""
+        """Performs a greedy linear search - moves the continuous features in CFs towards original values in
+           query_instance greedily until the prediction class changes."""
 
         old_diff = diff
-        change = (10**-decimal_prec[feature]) # the minimal possible change for a feature
+        change = (10**-decimal_prec[feature])  # the minimal possible change for a feature
         current_pred = current_pred_orig
         if self.model.model_type == 'classifier':
-            while((abs(diff)>10e-4) and (np.sign(diff*old_diff) > 0) and self.is_cf_valid(current_pred)):
+            while((abs(diff) > 10e-4) and (np.sign(diff*old_diff) > 0) and self.is_cf_valid(current_pred)):
                 old_val = final_cfs_sparse[feature].iat[cf_ix]
                 final_cfs_sparse.loc[cf_ix, feature] += np.sign(diff)*change
                 current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.iloc[[cf_ix]][self.data_interface.feature_names])
@@ -373,7 +394,8 @@ class ExplainerBase:
         return final_cfs_sparse
 
     def do_binary_search(self, diff, decimal_prec, query_instance, cf_ix, feature, final_cfs_sparse, current_pred):
-        """Performs a binary search between continuous features of a CF and corresponding values in query_instance until the prediction class changes."""
+        """Performs a binary search between continuous features of a CF and corresponding values
+           in query_instance until the prediction class changes."""
 
         old_val = final_cfs_sparse[feature].iat[cf_ix]
         final_cfs_sparse.loc[cf_ix, feature] = query_instance[feature].iat[0]
@@ -443,12 +465,11 @@ class ExplainerBase:
             self.target_cf_range = self.infer_target_cfs_range(desired_range)
         return desired_class
 
-    def infer_target_cfs_class(self, desired_class_input, original_pred,
-            num_output_nodes):
+    def infer_target_cfs_class(self, desired_class_input, original_pred, num_output_nodes):
         """ Infer the target class for generating CFs. Only called when
-        model_type=="classifier".
-        TODO: Add support for opposite desired class in multiclass. Downstream methods should decide
-        whether it is allowed or not.
+            model_type=="classifier".
+            TODO: Add support for opposite desired class in multiclass. Downstream methods should decide
+                  whether it is allowed or not.
         """
         if desired_class_input == "opposite":
             if num_output_nodes == 2:
@@ -456,7 +477,8 @@ class ExplainerBase:
                 target_class = int(1 - original_pred_1)
                 return target_class
             elif num_output_nodes > 2:
-                raise UserConfigValidationException("Desired class cannot be opposite if the number of classes is more than 2.")
+                raise UserConfigValidationException(
+                    "Desired class cannot be opposite if the number of classes is more than 2.")
         elif isinstance(desired_class_input, int):
             if desired_class_input >= 0 and desired_class_input < num_output_nodes:
                 target_class = desired_class_input
@@ -483,10 +505,12 @@ class ExplainerBase:
         for i in range(len(model_outputs)):
             pred = model_outputs[i]
             if self.model.model_type == "classifier":
-                if self.num_output_nodes == 2: # binary
+                if self.num_output_nodes == 2:  # binary
                     pred_1 = pred[self.num_output_nodes-1]
-                    validity[i] = 1 if ((self.target_cf_class == 0 and pred_1<= self.stopping_threshold) or (self.target_cf_class == 1 and pred_1>= self.stopping_threshold)) else 0
-                else: # multiclass
+                    validity[i] = 1 if \
+                        ((self.target_cf_class == 0 and pred_1 <= self.stopping_threshold) or
+                         (self.target_cf_class == 1 and pred_1 >= self.stopping_threshold)) else 0
+                else:  # multiclass
                     if np.argmax(pred) == self.target_cf_class:
                         validity[i] = 1
             elif self.model.model_type == "regressor":
@@ -514,11 +538,15 @@ class ExplainerBase:
 
             if self.num_output_nodes == 1:  # for tensorflow/pytorch models
                 pred_1 = model_score[0]
-                validity = True if ((target_cf_class == 0 and pred_1<= self.stopping_threshold) or (target_cf_class == 1 and pred_1>= self.stopping_threshold)) else False
+                validity = True if \
+                    ((target_cf_class == 0 and pred_1 <= self.stopping_threshold) or
+                     (target_cf_class == 1 and pred_1 >= self.stopping_threshold)) else False
                 return validity
             if self.num_output_nodes == 2:  # binary
                 pred_1 = model_score[self.num_output_nodes-1]
-                validity = True if ((target_cf_class == 0 and pred_1<= self.stopping_threshold) or (target_cf_class == 1 and pred_1>= self.stopping_threshold)) else False
+                validity = True if \
+                    ((target_cf_class == 0 and pred_1 <= self.stopping_threshold) or
+                     (target_cf_class == 1 and pred_1 >= self.stopping_threshold)) else False
                 return validity
             else:  # multiclass
                 if np.argmax(model_score) == target_cf_class:
@@ -542,13 +570,15 @@ class ExplainerBase:
                 model_output[i] = model_scores[i]
         return model_output
 
-    def check_permitted_range(self, permitted_range): # TODO: add comments as to where this is used if this function is necessary, else remove.
-        """checks permitted range for continuous features"""
+    def check_permitted_range(self, permitted_range):
+        """checks permitted range for continuous features
+           TODO: add comments as to where this is used if this function is necessary, else remove.
+        """
         if permitted_range is not None:
-        #     if not self.data_interface.check_features_range(permitted_range):
-        #         raise ValueError(
-        #             "permitted range of features should be within their original range")
-        #     else:
+            # if not self.data_interface.check_features_range(permitted_range):
+            #   raise ValueError(
+            #       "permitted range of features should be within their original range")
+            # else:
             self.data_interface.permitted_range = permitted_range
             self.minx, self.maxx = self.data_interface.get_minx_maxx(normalized=True)
             self.cont_minx = []
@@ -557,13 +587,16 @@ class ExplainerBase:
                 self.cont_minx.append(self.data_interface.permitted_range[feature][0])
                 self.cont_maxx.append(self.data_interface.permitted_range[feature][1])
 
-    def check_mad_validity(self, feature_weights): # TODO: add comments as to where this is used if this function is necessary, else remove.
-        """checks feature MAD validity and throw warnings"""
+    def check_mad_validity(self, feature_weights):
+        """checks feature MAD validity and throw warnings.
+           TODO: add comments as to where this is used if this function is necessary, else remove.
+        """
         if feature_weights == "inverse_mad":
             self.data_interface.get_valid_mads(display_warnings=True, return_mads=False)
 
-    def sigmoid(self, z): # used in VAE-based CF explainers
-            return 1 / (1 + np.exp(-z))
+    def sigmoid(self, z):
+        """This is used in VAE-based CF explainers."""
+        return 1 / (1 + np.exp(-z))
 
     def build_KD_tree(self, data_df_copy, desired_range, desired_class, predicted_outcome_name):
         # Stores the predictions on the training data
