@@ -22,7 +22,7 @@ copyright = '2020, Ramaravind, Amit, Chenhao'
 author = 'Ramaravind, Amit, Chenhao'
 
 # The full version, including alpha/beta/rc tags
-release = '0.6'
+release = '0.6.1'
 
 
 # -- General configuration ---------------------------------------------------
@@ -30,14 +30,18 @@ release = '0.6'
 # Add any Sphinx extension module names here, as strings. They can be
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
 # ones.
-extensions = ['sphinx.ext.autodoc',
+extensions = [
+    'sphinx.ext.autodoc',
     'sphinx.ext.viewcode',
     'sphinx.ext.todo',
     'nbsphinx',
     'sphinx_rtd_theme'
 ]
 
-autodoc_mock_imports = ['numpy', 'pandas', 'matplotlib', 'os', 'tensorflow', 'random', 'collections', 'timeit', 'tensorflow.keras', 'sklearn', 'sklearn.model_selection.train_test_split', 'copy', 'IPython', 'IPython.display.display', 'collections', 'collections.OrderedDict', 'logging', 'torch', 'torchvision']
+autodoc_mock_imports = ['numpy', 'pandas', 'matplotlib', 'os', 'tensorflow', 'random', 'collections',
+                        'timeit', 'tensorflow.keras', 'sklearn', 'sklearn.model_selection.train_test_split',
+                        'copy', 'IPython', 'IPython.display.display', 'collections', 'collections.OrderedDict',
+                        'logging', 'torch', 'torchvision']
 
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['_templates']
@@ -75,4 +79,4 @@ if not on_rtd:
 # so a file named "default.css" will overwrite the builtin "default.css".
 html_static_path = []
 
-html_sidebars = { '**': ['globaltoc.html', 'relations.html', 'sourcelink.html', 'searchbox.html'] }
+html_sidebars = {'**': ['globaltoc.html', 'relations.html', 'sourcelink.html', 'searchbox.html']}
